# Copyright 2024-2026 The DNS-AID Authors
# SPDX-License-Identifier: Apache-2.0

"""OpenTelemetry integration for DNS-AID SDK (opt-in)."""

from dns_aid.sdk.telemetry.otel import TelemetryManager

__all__ = ["TelemetryManager"]
