# Copyright 2024-2026 The DNS-AID Authors
# SPDX-License-Identifier: Apache-2.0

"""Signal collection and storage."""

from dns_aid.sdk.signals.collector import SignalCollector

__all__ = ["SignalCollector"]
